/**
 * <h1>Clase que desarrolla las caractersticas grficas del robot.</h1> 
 * Implementa un robot circular pequeo con tres sensores empotrados de igual
 * color que es capaz de moverse por un laberinto siguiendo cierto algoritmo.
 * 
 * @author Matas Contreras, Francisco Frez, Damin Quiroz.
 * @version 2.0
 * @since 2018-04-30
 */

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;

import javax.swing.*;

public class RobotView extends JPanel {
	
	/** Constructor base de la clase que permite creacin de un nuevo objeto. */
	public RobotView() {

	}
	
	/**
	 * Mtodo que permite la visualizacin del robot en la ventana.
	 * 
	 * @param robot:
	 * 			Variable del tipo Robot que construye el robot. 
	 */
	public RobotView(Robot robot) {
		this.robot = robot;
	}

	/**
	 * Mtodo que indica la posicin del robot en el JPanel (ventana).
	 * 
	 * @param position:
	 * 			Variable del tipo Vector2D que indica posicin x y del robot. 
	 */
	public RobotView(Vector2D position) {
		this.position = position;
	}
	
	/**
	 * Mtodo que se encarga de pintar el robot y sus sensores dentro del JPanel.
	 * 
	 * @param g:
	 * 			Variable Graphics2D donde se dibuja el robot (valo rojo). 
	 */
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		Graphics2D g2 = (Graphics2D) g;
		g2.setPaint(Color.red);
		
		Vector2D forward = robot.getVelocity().getNormalize();
		Vector2D left = forward.turnLeft();
		Vector2D right = forward.turnRight();
		
		x = (int) this.robot.getPosition().getX();
		y = (int) this.robot.getPosition().getY();
		
		/* Direccin del robot */
		int velx,vely ;
		velx = (int)forward.getX();
		vely = (int)forward.getY();
		
		/* Robot Principal */
		g2.drawOval(x-robotRadius/2, y-robotRadius/2, robotRadius, robotRadius);
		g2.setColor(Color.blue);
		
		/* Forward Sensor */
		g2.drawOval(x+velx*robotRadius/2-sensorRadius/2,y+vely*robotRadius/2-sensorRadius/2,sensorRadius,sensorRadius);
		
		/* Left Sensor */
		g2.setColor(Color.cyan);
		velx = (int)left.getX();
		vely = (int)left.getY();
		g2.drawOval(x+velx*robotRadius/2-sensorRadius/2,y+vely*robotRadius/2-sensorRadius/2,sensorRadius,sensorRadius);
		
		/* Right Sensor */
		g2.setColor(Color.orange);
		velx = (int)right.getX();
		vely = (int)right.getY();
		g2.drawOval(x+velx*robotRadius/2-sensorRadius/2,y+vely*robotRadius/2-sensorRadius/2,sensorRadius,sensorRadius);
		
		/* Permite que no parpadee la figura pintada en el JPanel */
		this.repaint();
		this.validate();
	}

	private Robot robot;
	private int x,y;
	private Vector2D position;
	public static int robotRadius = 8;
	public static int sensorRadius = 2;
}
