/**
 * <h1>Sensor de distancia</h1> Crea un sensor de distancia utilizado para una
 * clase Robot. Permite sensar la distancia hacia alguna pared en un laberinto.
 * <p>
 * <b>Note:</b> Sin modificaciones desde la version anterior, entregada en la
 * Tarea 1 del ramo ELO329.
 * 
 * @author Matias Contreras, Francisco Frez, Damin Quiroz.
 * @version 1.0
 * @since 2018-04-13
 */

public class DistanceSensor {

	/**
	 * Constructor base utilizado, debe tener un Laberinto como parmetro de
	 * entrada.
	 * 
	 * @param min
	 *            Variable del tipo Maze la cual implementa un laberinto.
	 */

	public DistanceSensor(Maze min) {
		this.range = 3;
		this.inMaze = min;
		/* define parametros de los sensores */
	}

	/**
	 * Metodo que setea el rango de los sensores.
	 * 
	 * @param rang
	 *            Variable del tipo double que setea un rango.
	 */

	public void setRange(double rang) {
		this.range = rang;
		/* setea el rango de los sensores */
	}

	/**
	 * Metodo que setea la direccin en la que apuntara el sensor.
	 * 
	 * @param newDir
	 *            Parmetro del tipo Vector2D.
	 */

	public void setDirection(Vector2D newDir) {
		this.direction = newDir.getNormalize();
		/*
		 * Por defecto el sensor se ubica en la misma direccion que va la velocidad en
		 * el tiempo de insercion al robot
		 */
	}

	/**
	 * Metodo que setea el Robot al que ser acoplado el sensor de distancia.
	 * 
	 * @param robot
	 *            Parmetro de entrada tipo Robot, ser el robot al que esta
	 *            acoplado el sensor.
	 */

	public void setSensor(Robot robot) {
		this.master = robot;
		this.range = robot.getSRange();
		this.direction = master.getVelocity().getNormalize();
		this.position = master.getPosition();
		/*
		 * Se puede seleccionar la direccion en la que apuntara el sensor al momento de
		 * ubicarlo en el robot
		 */
	}

	/**
	 * Versin mejorada del mtodo setSensor(Robot). Permite setear el sensor en un
	 * robot, con una direccin definida.
	 * 
	 * @param robot
	 *            Robot al que se setea el sensor. Variable del tipo Robot.
	 * @param dir
	 *            Direccin en la que se pondr el sensor. Variable del tipo
	 *            Vector2D, debe ser un vector unitario, de no serlo , se normaliza
	 *            dentro del mtodo.
	 */

	public void setSensor(Robot robot, Vector2D dir) {
		this.master = robot;
		this.position = master.getPosition();
		if (dir.getModule() != 1.0)
			this.direction = dir.getNormalize();
		else
			this.direction = dir;
	}

	/**
	 * Mtodo que retorna la distancia hacia la pared mas cercana dentro del rango
	 * del sensor.
	 * 
	 * @return double distancia : Distancia hasta la pared ms cercana, dentro del
	 *         rango del sensor, de no encontrar ninguna pared dentro de su rango ,
	 *         retorna -1. Si hay un error con la condicion de los sensores, retorna
	 *         la condicin de error -2.
	 */

	public double getDistance() {
		boolean[][] mazeArray = inMaze.getArray();
		int x, y, dis = 0;
		x = (int) position.getX();
		y = (int) position.getY();

		if (this.isUp()) {
			for (int i = 1; i <= range + 1; i++) {
				if (y - i == range) {
					return dis;
				} else {
					/* itera range +1 veces, porque dis va de 0 a range */
					if (mazeArray[y - i][x] == true)
						return dis;
					else if (i == range + 1)
						return -1;
					else
						dis += 1;
				}
			}
		} else if (this.isDown()) {
			for (int i = 1; i <= range + 1; i++) {
				if (y + i == master.getMazeHeight()) {
					return dis;
					/* itera range +1 veces, porque dis va de 0 a range */
				} else {
					if (mazeArray[y + i][x] == true)
						return dis;
					else if (i == range + 1)
						return -1;
					else
						dis += 1;
				}
			}
		} else if (this.isLeft()) {
			for (int i = 1; i <= range + 1; i++) {
				if (x - i == 0) {
					return dis;
				} else {
					/* itera range +1 veces, porque dis va de 0 a range */
					if (mazeArray[y][x - i] == true)
						return dis;
					else if (i == range + 1)
						return -1;
					else
						dis += 1;
				}
			}
		} else if (this.isRight()) {
			for (int i = 1; i <= range + 1; i++) {
				if (x + i == master.getMazeWidth()) {
					return dis;
				} else {
					/* itera range +1 veces, porque dis va de 0 a range */
					if (mazeArray[y][x + i] == true)
						return dis;
					else if (i == range + 1)
						return -1;
					else
						dis += 1;
				}
			}
		}
		return -2; /* condicion de error */
		/* obtiene la distancia del robot con las paredes */
	}

	/**
	 * Mtodo que sirve para cambiar la posicin del sensor en una direccin
	 * definida. Basicamente implementa una suma cartesiana entre un vector
	 * velocidad y la posicin del sensor.
	 * 
	 * @param dir
	 *            Direccin en la que se mueve el sensor.
	 */
	public void moveSensor(Vector2D dir) {
		this.position = position.plus(dir);
		/* mueve el sensor en conjunto con el robot */
	}

	/**
	 * Mtodo que retorna la direccin en la que est apuntando el sensor
	 * actualmente.
	 * 
	 * @return Vector2D direction.
	 */
	public Vector2D getDirection() {
		return this.direction;
		/* retorna la direccion del sensor */
	}

	/**
	 * Mtodo que retorna la posicin actual del sensor. Esta posicin est definida
	 * por la posicin en la que se encuentra el Robot en que se encuetre el sensor.
	 * 
	 * @return Vector2D position : Posicin actual del sensor.
	 */
	public Vector2D getPosition() {
		return this.position;
		/* retorna la posicion del sensor */
	}

	/**
	 * Mtodo que gira el sensor hacia su izquierda. ste mtodo sirve cuando el
	 * Robot cambia su sentido de giro, por lo tanto el sensor debe moverse con l.
	 * <b>Nota:</b> El plano utilizado para calcular el giro, est centrado en el
	 * punto (0,0), con su eje X apuntando hacia la derecha y su eje Y positivo
	 * apuntando hacia abajo.
	 */
	public void turnRight() {
		double x, y;
		Vector2D vec = this.getDirection();
		x = vec.getX();
		y = vec.getY();
		this.direction.setTo(-y, x);
		/* giro a la derecha */
	}

	/**
	 * Mtodo que gira el sensor hacia su derecha. ste mtodo sirve cuando el Robot
	 * cambia su sentido de giro, por lo tanto el sensor debe moverse con l.
	 * <b>Nota:</b> El plano utilizado para calcular el giro, est centrado en el
	 * punto (0,0), con su eje X apuntando hacia la derecha y su eje Y positivo
	 * apuntando hacia abajo.
	 */
	public void turnLeft() {
		double x, y;
		Vector2D vec = this.getDirection();
		x = vec.getX();
		y = vec.getY();
		this.direction.setTo(y, -x);
		/* giro a la izquierda */
	}

	/**
	 * Mtodo que indica si el sensor est apuntando hacia la derecha en el
	 * laberinto.
	 * 
	 * @return boolean isRight : Toma el valor true si el sensor apunta hacia la
	 *         derecha del laberinto, si no, retorna false.
	 */
	public boolean isRight() {
		if (this.direction.getX() == 1 && this.direction.getY() == 0)
			return true;
		else
			return false;
		/* sensor apunta hacia la derecha */
	}

	/**
	 * Mtodo que indica si el sensor est apuntando hacia la izquierda en el
	 * laberinto.
	 * 
	 * @return boolean isRight : Toma el valor true si el sensor apunta hacia la
	 *         izquierda del laberinto, si no, retorna false.
	 */
	public boolean isLeft() {
		if (this.direction.getX() == -1 && this.direction.getY() == 0)
			return true;
		else
			return false;
		/* sensor apunta hacia la izquierda */
	}

	/**
	 * Mtodo que indica si el sensor est apuntando hacia arriba en el laberinto.
	 * 
	 * @return boolean isRight : Toma el valor true si el sensor apunta hacia arriba
	 *         del laberinto, si no, retorna false.
	 */
	public boolean isUp() {
		if (this.direction.getX() == 0 && this.direction.getY() == -1)
			return true;
		else
			return false;
		/* sensor apunta hacia arriba */
	}

	/**
	 * Mtodo que indica si el sensor est apuntando hacia abajo en el laberinto.
	 * 
	 * @return boolean isRight : Toma el valor true si el sensor apunta hacia abajo
	 *         del laberinto, si no, retorna false.
	 */
	public boolean isDown() {
		if (this.direction.getX() == 0 && this.direction.getY() == 1)
			return true;
		else
			return false;
		/* sensor apunta hacia abajo */
	}

	private double range;
	private Maze inMaze;
	private Robot master;
	private Vector2D direction;
	private Vector2D position;
}
