/**
 * <h1>Laberinto binario</h1> Crea una matriz laberinto binaria mediante un archivo externo del tipo PBM.
 * <p>
 * <b>Note:</b> Presenta modificaciones respecto a la version anterior utilizada en el Stage2 de la Tarea 2 del ramo
 * ELO329. Se eliminan Metodos no usados e implementacion de nuevos metodos.
 * 
 * @author Matias Contreras, Francisco Frez, Dami�n Quiroz.
 * @version 2.0
 * @since 2018-04-30
 */

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.geom.*;

public class Maze {
	
	/** Constructor base de la clase, deja el arreglo laberinto apuntando a null para evitar excepciones.*/
	public Maze() {
		this.array = null;
		/* inicializa el laberinto */
	}
	/** Constructor secundario de la clase. Recibe una variable tipo Maze de la cual copia sus parametros y 
	 * su laberinto binario.
	 * @param in : Variable tipo maze de la cual se copian los atributos.*/
	public Maze(Maze in) {
		boolean[][] aux = in.getArray();
		int waux, haux;
		waux = in.getWidth();
		haux = in.getHeight();
		this.hight = haux;
		this.width = waux;

		/*
		 * Se cambio el metodo en que se copia el Maze de entrada, ya que al copiar la
		 * referencia se sobreescribia el archivo de salida
		 */

		this.array = new boolean[hight][width];
		for (int h = 0; h < hight; h++) {
			for (int w = 0; w < width; w++) {
				this.array[h][w] = aux[h][w];
			}
		}
	}
	
	
	/** Constructor terciario de la clase. Recibe una variable tipo scanner, la cual apunta a un archivo de extension PBM.
	 * @param sc : Variable tipo Scanner que contiene la ruta de un archivo PBM desde el que se adquieren los tributos y el laberinto.*/
	public Maze(Scanner sc) {
		String s;
		s = sc.nextLine();
		while (s.charAt(0) != '#') {
			s = sc.nextLine();
			this.state = s;
		}
		while (!sc.hasNextInt())
			sc.nextLine();
		width = sc.nextInt();
		hight = sc.nextInt();
		/* lee las primeras 2 lineas para saber las dimensiones de la matriz */

		sc.nextLine();
		array = new boolean[hight][];
		/* Crea un arreglo booleano de alto (filas) hight */
		for (int h = 0; h < hight; h++)
			/* las filas tendran ancho width */
			array[h] = new boolean[width];
		for (int h = 0; h < hight; h++)
			for (int w = 0; w < width;) {
				/* sirve para ver si hay divisiones y rellenar la matriz */
				s = sc.findInLine(".");
				if (s == null)
					sc.nextLine();
				else
					array[h][w++] = (s.charAt(0) == '1');
			}
	}
	
	/** Metodo que escribe sobre un laberinto la ruta de un objeto tipo Robot.
	 * @param inRobot : Variable tipo Robot desde la que se toma un Maze y se sobreescribe.*/
	public void drawRoute(Robot inRobot) {
		int width, height;
		width = (int) inRobot.getMazeWidth();
		height = (int) inRobot.getMazeHeight();
		boolean[][] rArray = inRobot.getRouteArray();
		for (int i = 0; i < height; i++) {
			for (int k = 0; k < width; k++) {
				this.array[i][k] |= rArray[i][k];
			}
		}
		/* dibuja la ruta seguida en el laberinto */
	}
	
	/** Metodo que entrega el Arreglo que contiene el laberinto binario.
	 * @return boolean[][] array : Arreglo binario que contiene una estructura de laberinto.*/
	public boolean[][] getArray() {
		boolean[][] aux = new boolean[this.getHeight()][this.getWidth()];
		for (int h = 0; h < hight; h++)
			for (int w = 0; w < width;w++) {
				aux[h][w] = this.array[h][w];
			}
		return aux;
		/* obtiene el arreglo multidimensional (laberinto) */
	}
	
	/** Metodo que entrega una variable tipo Maze con los parametros del laberinto, con lo que se implementa una copia del mismo.
	 * @return Maze nMaze : Variable tipo Maze con todos los atributos del laberinto base.*/
	public Maze clone() {
		Maze nMaze = new Maze();
		boolean [][] aux = new boolean[hight][width];
		aux = this.getArray();
		 nMaze.setArray(aux);
		return nMaze;
	}
	
	/** Metodo que se utiliza para darle valores al arreglo booleano que implementa el laberinto, junto con sus parametros.
	 * @param inArray : Variable del tipo boolean[][] que contiene un arreglo booleano con una estructura de laberinto.
	 */
	public void setArray(boolean[][] inArray) {
		this.array = inArray;
		this.width = array[0].length;
		this.hight = array.length;
	}
	
	/** Metodo que se utiliza para obtener el parametro "alto" del laberinto.
	 * @return int height : Retorna un valor entero con el alto del arreglo booleando que implementa el laberinto.*/
	public int getHeight() {
		return this.hight;
		/* obtiene la altura del laberinto */
	}
	/**
	 * Metodo que se utiliza para obtener el "ancho" del laberinto.
	 * @return int width: Retorna un valor entero con el alto del arreglo booleano que implementa el laberinto.
	 * */
	public int getWidth() {
		return this.width;
		/* obtiene el ancho del laberinto */
	}
	
	/** 
	 * Metodo que se utiliza para escribir  el laberinto desde el arreglo booleano, a un archivo externo con todos su valores.
	 * @param out : Variable del tipo PrintStream que sirve para enviar los datos del laberinto a un archivo.*/
	public void write(PrintStream out) {
		out.println("P1");
		out.println("#Created by " + getClass().getName() + "UTFSM ELO329");
		out.println(width + " " + hight);
		for (int i = 0; i < hight; i++) {
			String s = new String();
			for (int j = 0; j < width; j++) {
				if (array[i][j] == true)
					s += "1";
				else
					s += "0";
			}
			out.println(s);
			/* imprime los valores del laberinto y los necesarios */
		}
	}
	
	/** Metodo utilizado para dibujar el laberinto en un JPanel.
	 * @param g : variable del tipo Graphics2D que implementa el metodo paintComponent de forma interna, el cual sirve para dibujar
	 * elementos en un JPanel.*/
	public void draw(Graphics2D g) {
		for (int h = 0; h < array.length; h++) {
			for (int w = 0; w < array[0].length; w++) {
				if (this.array[h][w] == true)
					g.draw(new Line2D.Double(w, h, w, h));
			}
		}

	}

	private boolean[][] array;
	private int width, hight;
	String state;
	/* define variables necesarias y su visibilidad */
}