/**
 * <h1>Clase que extiende de Pilot y define el algoritmo de otro piloto.</h1> 
 * Implementa constructores y m�todos necesarios para definir el algoritmo "pared 
 * a la izquierda" (avanza hasta que encuentra un pared a la izquierda y luego se 
 * mueve pegado a ella). 
 * 
 * @author Mat�as Contreras, Francisco Frez, Dami�n Quiroz.
 * @version 1.0
 * @since 2018-04-30
 */


public class MyPilot extends Pilot {
	
	/** Constructor base de la clase, se inicializan las variables creadas. 
	 * @param in : Variable del tipo Robot. Setea el Robot desde el que se tomarán los datos y se manejará.*/
	MyPilot(Robot in) {
		super(in);
	}
	
	/** Constructor secundario de la clase. Se setea el tipo de piloto a usar.
	 * @param in: Variable del tipo Robot. Setea el Robot desde el que se tomarán datos y se manejará.
	 * @param stype : Variable del tipo String que setea el tipo de piloto a implementar. <b>Nota:<b> Los valores de stype pueden ser
	 * "A" o "a", para un piloto  que usa la pared izquierda como referencia, y "P" o "p" para un piloto que usa la pared derecha como referencia.*/
	MyPilot(Robot in,String stype){
		super(in);
		this.sType = stype;
		
	}
	
	/**
	 * M�todo que implementa los algoritmos necesarios para salir del laberinto.
	 * 
	 * @param delta_t:
	 * 			Variable del tipo double que define el tiempo al interior de 
	 * la simulaci�n (permite generaci�n de movimiento).
	 * <b>Nota:</b> define una serie de variables auxiliares que sirven de
	 * apoyo para el cumplimiento de las m�ltiples condiciones implementadas. 
	 */
	public void setCourse(double delta_t) {
		int lastR = (int) super.rRigth.getDistance();
		int lastF = (int) super.rForward.getDistance();
		int lastL = (int) super.rLeft.getDistance();

		/* Variables booleanas que permiten consultar el estado del robot */

		if (sType == "A" || sType == "a") {
			boolean PDer, PIzq, PFor, PLost, PFil, PMedR, PMedL, PMedF;

			PMedR = super.posRange.contains(lastR);
			PMedL = super.posRange.contains(lastL);
			PMedF = super.posRange.contains(lastF);

			PDer = ((PMedR) && !(PMedF) && !(PMedL));
			PIzq = (!(PMedR) && !(PMedF) && (PMedL));
			PFor = (!(PMedR) && (PMedF) && !(PMedL));
			PLost = (!(PMedR) && !(PMedF) && !(PMedL));
			PFil = (!(PMedR) && (PMedF) && (PMedL));

			/*
			 * Algoritmo para resolver el laberinto usando la pared izquierda como
			 * referencia
			 */

			/*
			 * Primera condicion permite que los sensores del robot no sensen fuera del
			 * laberinto y ademas que el robot no salga del laberinto
			 */

			if (PDer || PFor || PFil) {
				super.stateCont = 0;
				super.robot.traceRoute();
				super.robot.turnRight();
			} else if (PIzq) {
				super.stateCont = 0;
				super.robot.traceRoute();
				super.robot.moveDelta_t(delta_t);
			} else if (PLost) {
				super.robot.traceRoute();
				if (stateCont == 0)
					robot.turnLeft();
				else if (stateCont == 1)
					robot.moveDelta_t(delta_t);
				else if (stateCont == 2)
					robot.turnRight();
				else if (stateCont == 3)
					robot.moveDelta_t(delta_t);
				else
					System.err.println("Error de sistema autodestruccion inminente, me pase poh!");

				stateCont = (stateCont + 1) % 4;

				/*
				 * Si entra en la condicion por defecto, quiere decir que se seteo el robot en
				 * un lugar indebido, o esta en una condicion no prevista.
				 */

			} else {
				robot.traceRoute();
				robot.getPosition();
				System.err.println("Save me pls!");
			}
		} else {
			boolean PDer, PIzq, PFor, PLost, PFir, PMedR, PMedL, PMedF;
			PMedR = posRange.contains(lastR);
			PMedL = posRange.contains(lastL);
			PMedF = posRange.contains(lastF);

			PDer = ((PMedR) && !(PMedF) && !(PMedL));
			PIzq = (!(PMedR) && !(PMedF) && (PMedL));
			/* (lastR !=2 && lastF !=2 && lastL ==2); */
			PFor = (!(PMedR) && (PMedF) && !(PMedL));
			/* (lastR !=2 && lastF == 2 && lastL !=2); */
			PLost = (!(PMedR) && !(PMedF) && !(PMedL));
			/* (lastR !=2 && lastF != 2 && lastL !=2); */
			PFir = ((PMedR) && (PMedF) && !(PMedL));
			/* (lastR == }else2 && lastF == 2 && lastL !=2); */

			/*
			 * Algoritmo para resolver el laberinto usando la pared derecha como referencia
			 */

			/*
			 * Primera condicion permite que los sensores del robot no sensen fuera del
			 * laberinto y ademas que el robot no salga del laberinto.
			 */

			if (PDer) {
				stateCont = 0;
				robot.traceRoute();
				robot.moveDelta_t(delta_t);
			} else if (PIzq || PFor || PFir) {
				stateCont = 0;
				robot.traceRoute();
				robot.turnLeft();
			} else if (PLost) {
				robot.traceRoute();
				if (stateCont == 0)
					robot.turnRight();
				else if (stateCont == 1)
					robot.moveDelta_t(delta_t);
				else if (stateCont == 2)
					robot.turnLeft();
				else if (stateCont == 3)
					robot.moveDelta_t(delta_t);
				else
					System.err.println("Me pase poh");

				stateCont = (stateCont + 1) % 4;

				/*
				 * Si entra en la condicion por defecto, quiere decir que se seteo el robot en
				 * un lugar indebido, o esta en una condicion no prevista.
				 */

			} else {
				robot.traceRoute();
				robot.getPosition().setTo(225, 190);
				System.err.println("Save me pls!");
			}
		}
	}
	String sType;
}
