
/**
 * <h1> Dibujo de Robot en laberinto </h1> Crea la silueta de un robot con tres sensores de distancia. El cual se setea en un laberinto grafico.
 * <p>
 * <b>Note:</b> Segunda version de la clase, modificaciones necesarias para utilizar el mouse a la hora de crear un nuevo robot.
 * 
 * @author Matias Contreras, Francisco Frez, Dami�n Quiroz.
 * @version 2.0
 * @since 2018-04-30
 */


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class RobotView extends JPanel {
	/**
	 * Constructor base de la clase, solo implementa la creacion de un nuevo objeto.
	 */
	public RobotView() {

	}
	/**
	 * Constructor secundario de la clase, implementa la creacion de un objeto y
	 * setea un robot a la clase.
	 * 
	 * @param robot
	 *            : Variable del tipo robot. Guarda un robot del cual se obtiene los
	 *            parametros para dibujarlo en el laberinto.
	 */
	public RobotView(Robot robot) {
		this.robot = robot;
		mouseMotionHandler = new MouseMotionHandler();
		this.addMouseMotionListener(mouseMotionHandler);
		this.addMouseListener(new MouseHandler(this, mouseMotionHandler));
	}
	
	/** Metodo utilizado para dibujar en pantalla un Robot.*/

	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		Graphics2D g2 = (Graphics2D) g;
		g2.setPaint(Color.red);
		if (robot.getPosition().getX() == 0.0 && robot.getPosition().getY() == 0.0) {

			/* Dibujo del roboy */
			Vector2D forward = robot.getVelocity().getNormalize();
			Vector2D left = forward.turnLeft();
			Vector2D right = forward.turnRight();
			g2.fillOval(x - robotRadius / 2, y - robotRadius / 2, robotRadius, robotRadius);
			g2.drawOval(x - robotRadius / 2, y - robotRadius / 2, robotRadius, robotRadius);
			int velx, vely;
			velx = (int) forward.getX();
			vely = (int) forward.getY();
			/* Robot principal */
			g2.fillOval(x - robotRadius / 2, y - robotRadius / 2, robotRadius, robotRadius);
			g2.drawOval(x - robotRadius / 2, y - robotRadius / 2, robotRadius, robotRadius);
			g2.setColor(Color.blue);
			/* Forward sensor */
			g2.fillOval(x + velx * robotRadius / 2 - sensorRadius / 2, y + vely * robotRadius / 2 - sensorRadius / 2,
					sensorRadius, sensorRadius);
			g2.drawOval(x + velx * robotRadius / 2 - sensorRadius / 2, y + vely * robotRadius / 2 - sensorRadius / 2,
					sensorRadius, sensorRadius);
			/* left Sensor */

			velx = (int) left.getX();
			vely = (int) left.getY();
			g2.fillOval(x + velx * robotRadius / 2 - sensorRadius / 2, y + vely * robotRadius / 2 - sensorRadius / 2,
					sensorRadius, sensorRadius);
			g2.drawOval(x + velx * robotRadius / 2 - sensorRadius / 2, y + vely * robotRadius / 2 - sensorRadius / 2,
					sensorRadius, sensorRadius);
			/* right sensor */

			velx = (int) right.getX();
			vely = (int) right.getY();
			g2.fillOval(x + velx * robotRadius / 2 - sensorRadius / 2, y + vely * robotRadius / 2 - sensorRadius / 2,
					sensorRadius, sensorRadius);
			g2.drawOval(x + velx * robotRadius / 2 - sensorRadius / 2, y + vely * robotRadius / 2 - sensorRadius / 2,
					sensorRadius, sensorRadius);
			this.repaint();
			this.revalidate();
		} else {

			/* Algoritmo para detectar un muro */
			DistanceSensor forw = robot.getSensor("f");
			DistanceSensor lef = robot.getSensor("l");
			DistanceSensor righ = robot.getSensor("r");
			boolean fwall, lwall, rwall;
			/*-1 es nuestra condicion de no deteccion de paredes */
			fwall = forw.getDistance() != -1;
			lwall = lef.getDistance() != -1;
			rwall = righ.getDistance() != -1;
			
			
			Vector2D forward = robot.getVelocity().getNormalize();
			Vector2D left = forward.turnLeft();
			Vector2D right = forward.turnRight();

			int xp = (int) this.robot.getPosition().getX();
			int yp = (int) this.robot.getPosition().getY();
			/* Direccion del robot */
			int velx, vely;
			velx = (int) forward.getX();
			vely = (int) forward.getY();
			/* Robot principal */
			g2.fillOval(xp - robotRadius / 2, yp - robotRadius / 2, robotRadius, robotRadius);
			g2.drawOval(xp - robotRadius / 2, yp - robotRadius / 2, robotRadius, robotRadius);
			g2.setColor(Color.blue);
			/* Forward sensor */
			
			if(fwall) {
				g2.setColor(Color.green);
			}else {
				g2.setColor(Color.blue);
			}
			g2.fillOval(xp + velx * robotRadius / 2 - sensorRadius / 2, yp + vely * robotRadius / 2 - sensorRadius / 2,
					sensorRadius, sensorRadius);
			g2.drawOval(xp + velx * robotRadius / 2 - sensorRadius / 2, yp + vely * robotRadius / 2 - sensorRadius / 2,
					sensorRadius, sensorRadius);
			/* left Sensor */

			velx = (int) left.getX();
			vely = (int) left.getY();
			if(lwall) {
				g2.setColor(Color.green);
			}else {
				g2.setColor(Color.blue);
			}
			g2.fillOval(xp + velx * robotRadius / 2 - sensorRadius / 2, yp + vely * robotRadius / 2 - sensorRadius / 2,
					sensorRadius, sensorRadius);
			g2.drawOval(xp + velx * robotRadius / 2 - sensorRadius / 2, yp + vely * robotRadius / 2 - sensorRadius / 2,
					sensorRadius, sensorRadius);
			/* right sensor */
			velx = (int) right.getX();
			vely = (int) right.getY();
			if(rwall) {
				g2.setColor(Color.green);
			}else {
				g2.setColor(Color.blue);
			}
			g2.fillOval(xp + velx * robotRadius / 2 - sensorRadius / 2, yp + vely * robotRadius / 2 - sensorRadius / 2,
					sensorRadius, sensorRadius);
			g2.drawOval(xp + velx * robotRadius / 2 - sensorRadius / 2, yp + vely * robotRadius / 2 - sensorRadius / 2,
					sensorRadius, sensorRadius);
			robot.getWorld().run();
			this.repaint();
			this.revalidate();
		}
	}
	
	/** Clase interna que se utiliza para detectar el movimiento del mouse.*/
	private class MouseMotionHandler implements MouseMotionListener {
		public void mouseMoved(MouseEvent event) {
			x = event.getX() / 2;
			y = event.getY() / 2;
		}

		public void mouseDragged(MouseEvent event) {
			x = event.getX() / 2;
			y = event.getY() / 2;
		}
	}
	
	/** Clase interna que se utiliza para detectar el click del mouse.*/
	private class MouseHandler extends MouseAdapter {
		public MouseHandler(RobotView rView, MouseMotionHandler mouseMotionHandler) {
			this.rView = rView;
			this.mouseMotionHandler = mouseMotionHandler;
		}

		public void mousePressed(MouseEvent event) {

		}

		public void mouseClicked(MouseEvent event) {
			robot.getPosition().setTo(event.getX() / 2, event.getY() / 2);
			rView.removeMouseListener(this);
			rView.removeMouseMotionListener(mouseMotionHandler);
		}

		private RobotView rView;
		private MouseMotionHandler mouseMotionHandler;
	}

	private Robot robot;
	private int x;
	private int y;
	public static int robotRadius = 8;
	public static int sensorRadius = 2;
	private MouseMotionHandler mouseMotionHandler;
}
