
/**
 * <h1>Ventana para la configuraci�n del delta t.</h1> 
 * Implementa una ventana donde se puede definir el valor de delta t. 
 * <p>
 * <b>Nota:</b> Esta clase es separada en un archivo aparte desde el Stage3.
 * 
 * @author Mat�as Contreras, Francisco Frez, Dami�n Quiroz.
 * @version 2.0
 * @since 2018-05-01
 */

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;


class DeltaTFrame extends JFrame {
	/**
	 * Constructor base de la clase. Crea la ventana con todas las figuras y entrada
	 * de texto para poder definir el valor de delta t.
	 * 
	 * @param inWorld:
	 *            Variable tipo MyWorld a la cual se ingresa el valor de delta t.
	 * 
	 */
	public DeltaTFrame(MyWorld inWorld) {
		this.world = inWorld;
		this.setTitle("Set delta t");
		this.setSize(DEFAULT_WIDTH, DEFAULT_HEIGHT);

		/* Obtiene dimensiones de pantalla */
		Toolkit myScreen = Toolkit.getDefaultToolkit();
		Dimension screenSize = myScreen.getScreenSize();

		int widthScreen = screenSize.width;
		int heigthScreen = screenSize.height;

		/* Ventana inicia en el centro */
		this.setLocation(widthScreen / 2 - DEFAULT_WIDTH / 2, heigthScreen / 2 - DEFAULT_HEIGHT / 2);

		/* Icono de la ventana */
		Image negroIcono = myScreen.getImage("images/icono_negro.png");
		this.setIconImage(negroIcono);

		/* Panel principal del Frame */
		panel = new JPanel();
		this.add(panel);
		panel.setLayout(new BorderLayout());
		panel.setBackground(Color.WHITE);

		/* Descripcion de la ventana */
		JLabel label = new JLabel("Ingrese valor de delta t en segundos. Usar punto para decimales");
		label.setHorizontalAlignment(JLabel.CENTER);
		label.setFont(new Font("Italic", Font.BOLD, 16));
		panel.add(label, BorderLayout.NORTH);

		/* Panel que contiene el cuadro para ingresar los datos */
		deltaTPanel = new DeltaTPanel(this, panel, world);
		deltaTPanel.setBackground(Color.WHITE);
		panel.add(deltaTPanel, BorderLayout.CENTER);
	}

	/**
	 * M�todo para obtener el valor delta t.
	 * 
	 * @return deltaTPanel.getDeltaT(): Variable tipo float que contiene el valor de
	 *         delta t.
	 */
	public float getDeltaT() {
		return deltaTPanel.getDeltaT();
	}

	private MyWorld world;
	private static final int DEFAULT_WIDTH = 600;
	private static final int DEFAULT_HEIGHT = 200;
	private JPanel panel;
	private DeltaTPanel deltaTPanel;

}

/**
 * <h1>JPanel que se adhiere al DeltaTFrame.</h1> En este JPanel se implementa
 * el JTextField en el que se ingresa el valor de delta t y el mensaje de error
 * en caso de ingresar un dato no v�lido.
 * 
 */

class DeltaTPanel extends JPanel {
	/**
	 * Constructor Base de la clase. Crea la ventana con todas las figuras y entrada
	 * de texto para poder definir el valor de delta t.
	 * 
	 * @param frame:
	 *            Variable tipo DeltaTFrame. Es la ventana principal donde ocurre
	 *            todo.
	 * @param panel:
	 *            Variable tipo JPanel. Es la lamina donde se ingresan todas las
	 *            interfaces gr�ficas.
	 * @param inWorld:
	 *            Variable tipo MyWorld a la cual se ingresa el valor de delta t.
	 * 
	 */
	public DeltaTPanel(DeltaTFrame frame, JPanel panel, MyWorld inWorld) {
		this.world = inWorld;
		this.frame = frame;
		textField = new JTextField(20);
		this.add(textField);

		/* Boton para definir el valor de delta t */
		JButton deltaTButton = new JButton("Definir");

		/*
		 * Boton como listener a la espera de ser apretado. Al apretarse se cierrra la
		 * ventana
		 */
		DeltaValueListener deltaValueListener = new DeltaValueListener();
		deltaTButton.addActionListener(deltaValueListener);
		this.add(deltaTButton);

		/* Label para mensaje de error */
		labelPanel = new JPanel();
		labelPanel.setOpaque(false);
		labelPanel.setBackground(Color.WHITE);
		panel.add(labelPanel, BorderLayout.SOUTH);

		label = new JLabel("");
		label.setHorizontalAlignment(JLabel.CENTER);
		label.setFont(new Font("Italic", Font.BOLD, 16));
		labelPanel.add(label);
	}

	/**
	 * M�todo para obtener el valor delta t.
	 * 
	 * @return delta_t: Variable tipo float que contiene el valor de delta t.
	 */
	public float getDeltaT() {
		return delta_t;
	}

	/** Clase interna que implementa un ActionListener para el frame generado. */
	private class DeltaValueListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			try {
				/* Al definir el delta t, se almacena el valor y se cierra la ventana */
				delta_t = Float.parseFloat(textField.getText());
				frame.dispose(); // Como paso el frame, aqui lo cierro
				System.out.println("Se ha seteado el delta t en :" + delta_t);
				world.setDeltaT(delta_t);

			} catch (java.lang.NumberFormatException ex) {
				/* Mensaje de error en caso de recibir un dato erroneo */
				label.setText("Ingrese un valor valido para delta t");
			}
		}
	}

	private MyWorld world;
	private JTextField textField;
	private float delta_t;
	private DeltaTFrame frame;
	private JPanel labelPanel;
	private JLabel label;
}